"""
RSA加密算法实现封装
基于cryptography库
"""

import base64
from typing import Union, Optional
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
from cryptography.hazmat.primitives import serialization, hashes


class RSACipher:
    """
    RSA加密/解密和签名/验证封装类
    """

    def __init__(self, private_key=None, public_key=None):
        """
        使用可选密钥初始化RSA密码器
        
        :param private_key: PEM格式的私钥（可选）
        :param public_key: PEM格式的公钥（可选）
        """
        self.private_key = private_key
        self.public_key = public_key
        
        # Parse keys if provided
        self.private_key_obj: Optional[RSAPrivateKey] = None
        self.public_key_obj: Optional[RSAPublicKey] = None
        
        if private_key:
            if isinstance(private_key, str):
                private_key = private_key.encode('utf-8')
            key_obj = serialization.load_pem_private_key(
                private_key,
                password=None
            )
            if isinstance(key_obj, RSAPrivateKey):
                self.private_key_obj = key_obj
            else:
                raise ValueError("Provided key is not an RSA private key")
            
        if public_key:
            if isinstance(public_key, str):
                public_key = public_key.encode('utf-8')
            key_obj = serialization.load_pem_public_key(public_key)
            if isinstance(key_obj, RSAPublicKey):
                self.public_key_obj = key_obj
            else:
                raise ValueError("Provided key is not an RSA public key")

    @staticmethod
    def generate_keypair(key_size=2048):
        """
        生成新的RSA密钥对
        
        :param key_size: 密钥大小（比特），默认2048位
        :return: PEM格式的(私钥, 公钥)元组
        """
        # Generate private key
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=key_size,
        )
        
        # Get private key in PEM format
        private_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        
        # Get public key in PEM format
        public_key = private_key.public_key()
        public_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        )
        
        return private_pem.decode('utf-8'), public_pem.decode('utf-8')

    def encrypt(self, plaintext: str, public_key=None, padding_scheme='OAEP') -> str:
        """
        使用RSA算法加密明文
        
        :param plaintext: 要加密的文本
        :param public_key: PEM格式的公钥（可选，如果未提供则使用实例密钥）
        :param padding_scheme: 填充方案 ('OAEP', 'PKCS1', 'NONE')
        :return: Base64编码的加密密文
        """
        if not public_key and not self.public_key_obj:
            raise ValueError("Public key is required for encryption")
        
        if public_key:
            if isinstance(public_key, str):
                public_key = public_key.encode('utf-8')
            public_key_obj = serialization.load_pem_public_key(public_key)
            if not isinstance(public_key_obj, RSAPublicKey):
                raise ValueError("Provided key is not an RSA public key")
        else:
            public_key_obj = self.public_key_obj
            
        # Encrypt the plaintext
        if public_key_obj is None:
            raise ValueError("No valid public key available for encryption")
            
        # Prepare plaintext based on padding scheme
        plaintext_bytes = plaintext.encode('utf-8')
        
        # Apply padding scheme
        if padding_scheme == 'OAEP':
            pad = padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        elif padding_scheme == 'PKCS1':
            pad = padding.PKCS1v15()
        elif padding_scheme == 'NONE':
            # For no padding, manually implement raw RSA
            # This is not recommended for security reasons
            key_size = public_key_obj.key_size // 8
            # 检查明文长度是否合适
            if len(plaintext_bytes) > key_size - 11:
                raise ValueError("Plaintext too long for RSA encryption without padding")
            message_int = int.from_bytes(plaintext_bytes, 'big')
            encrypted_int = pow(message_int, public_key_obj.public_numbers().e, public_key_obj.public_numbers().n)
            return base64.b64encode(encrypted_int.to_bytes(key_size, 'big')).decode('utf-8')
        else:
            raise ValueError("Unsupported padding scheme. Use 'OAEP', 'PKCS1', or 'NONE'")
            
        ciphertext = public_key_obj.encrypt(plaintext_bytes, pad)
        
        # Return base64 encoded ciphertext
        return base64.b64encode(ciphertext).decode('utf-8')

    def decrypt_with_public_key(self, ciphertext: str, public_key=None, padding_scheme='OAEP') -> str:
        """
        使用RSA公钥解密密文（用于特殊情况，如服务器用私钥加密后返回给客户端）

        :param ciphertext: Base64编码的密文
        :param public_key: PEM格式的公钥（可选，如果未提供则使用实例密钥）
        :param padding_scheme: 填充方案 ('OAEP', 'PKCS1')
        :return: 解密后的明文
        """
        if not public_key and not self.public_key_obj:
            raise ValueError("Public key is required for decryption with public key")

        if public_key:
            if isinstance(public_key, str):
                public_key = public_key.encode('utf-8')
            public_key_obj = serialization.load_pem_public_key(public_key)
            if not isinstance(public_key_obj, RSAPublicKey):
                raise ValueError("Provided key is not an RSA public key")
        else:
            public_key_obj = self.public_key_obj

        # Decode base64 ciphertext
        ciphertext_bytes = base64.b64decode(ciphertext)

        # Decrypt the ciphertext
        if public_key_obj is None:
            raise ValueError("No valid public key available for decryption")

        # 注意：这里我们使用公钥进行"解密"操作，这在技术上是反向操作
        # 实际上这是使用公钥的指数和模数进行幂运算
        ciphertext_int = int.from_bytes(ciphertext_bytes, 'big')
        # 使用公钥的e指数进行解密操作
        decrypted_int = pow(ciphertext_int, public_key_obj.public_numbers().e, public_key_obj.public_numbers().n)

        # 转换回字节
        key_size = public_key_obj.key_size // 8
        decrypted_bytes = decrypted_int.to_bytes(key_size, 'big')

        # 移除填充（根据具体填充方案）
        if padding_scheme == 'PKCS1':
            # PKCS1 v1.5 解填充 - 更宽松的处理方式
            # 查找第一个 0x00 字节的位置
            zero_index = decrypted_bytes.find(0x00)
            if zero_index != -1:
                # 通常在 PKCS1 v1.5 中，数据以 0x00 0x02 开头，然后是随机填充，再以 0x00 分隔符结尾
                # 找到最后一个 0x00 分隔符
                last_zero_index = decrypted_bytes.rfind(0x00)
                if last_zero_index != -1 and last_zero_index < len(decrypted_bytes) - 1:
                    decrypted_bytes = decrypted_bytes[last_zero_index + 1:]
                else:
                    # 如果找不到合适的分隔符，就简单移除开头的零字节
                    decrypted_bytes = decrypted_bytes.lstrip(b'\x00')
            else:
                # 简单处理：移除开头的零字节
                decrypted_bytes = decrypted_bytes.lstrip(b'\x00')
        elif padding_scheme == 'OAEP':
            # OAEP 解填充比较复杂，这里简化处理
            decrypted_bytes = decrypted_bytes.lstrip(b'\x00')

        return decrypted_bytes.decode('utf-8', errors='ignore')

    def decrypt(self, ciphertext: str, private_key=None, padding_scheme='OAEP') -> str:
        """
        使用RSA算法解密密文
        
        :param ciphertext: Base64编码的密文
        :param private_key: PEM格式的私钥（可选，如果未提供则使用实例密钥）
        :param padding_scheme: 填充方案 ('OAEP', 'PKCS1')
        :return: 解密后的明文
        """
        if not private_key and not self.private_key_obj:
            raise ValueError("Private key is required for decryption")
            
        if private_key:
            if isinstance(private_key, str):
                private_key = private_key.encode('utf-8')
            private_key_obj = serialization.load_pem_private_key(
                private_key,
                password=None
            )
            if not isinstance(private_key_obj, RSAPrivateKey):
                raise ValueError("Provided key is not an RSA private key")
        else:
            private_key_obj = self.private_key_obj
            
        # Decode base64 ciphertext
        ciphertext_bytes = base64.b64decode(ciphertext)
            
        # Decrypt the ciphertext
        if private_key_obj is None:
            raise ValueError("No valid private key available for decryption")
            
        # Apply padding scheme
        if padding_scheme == 'OAEP':
            pad = padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        elif padding_scheme == 'PKCS1':
            pad = padding.PKCS1v15()
        elif padding_scheme == 'NONE':
            # For no padding, manually implement raw RSA decryption
            # This is not recommended for security reasons
            key_size = private_key_obj.key_size // 8
            ciphertext_int = int.from_bytes(ciphertext_bytes, 'big')
            decrypted_int = pow(ciphertext_int, private_key_obj.private_numbers().d, private_key_obj.private_numbers().public_numbers.n)
            plaintext_bytes = decrypted_int.to_bytes(key_size, 'big')
            # Remove leading zeros
            plaintext_bytes = plaintext_bytes.lstrip(b'\x00')
            return plaintext_bytes.decode('utf-8')
        else:
            raise ValueError("Unsupported padding scheme. Use 'OAEP', 'PKCS1', or 'NONE'")
            
        plaintext = private_key_obj.decrypt(ciphertext_bytes, pad)
        
        return plaintext.decode('utf-8')

    def sign(self, data: str, private_key=None, padding_scheme='PSS') -> str:
        """
        使用RSA算法对数据进行签名
        
        :param data: 要签名的数据
        :param private_key: PEM格式的私钥（可选，如果未提供则使用实例密钥）
        :param padding_scheme: 填充方案 ('PSS', 'PKCS1')
        :return: Base64编码的签名
        """
        if not private_key and not self.private_key_obj:
            raise ValueError("Private key is required for signing")
            
        if private_key:
            if isinstance(private_key, str):
                private_key = private_key.encode('utf-8')
            private_key_obj = serialization.load_pem_private_key(
                private_key,
                password=None
            )
            if not isinstance(private_key_obj, RSAPrivateKey):
                raise ValueError("Provided key is not an RSA private key")
        else:
            private_key_obj = self.private_key_obj
            
        # Sign the data
        if private_key_obj is None:
            raise ValueError("No valid private key available for signing")
            
        data_bytes = data.encode('utf-8')
        
        # Apply padding scheme
        if padding_scheme == 'PSS':
            pad = padding.PSS(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                salt_length=padding.PSS.MAX_LENGTH
            )
            algorithm = hashes.SHA256()
        elif padding_scheme == 'PKCS1':
            pad = padding.PKCS1v15()
            algorithm = hashes.SHA256()
        else:
            raise ValueError("Unsupported padding scheme. Use 'PSS' or 'PKCS1'")
            
        signature = private_key_obj.sign(data_bytes, pad, algorithm)
        
        # Return base64 encoded signature
        return base64.b64encode(signature).decode('utf-8')

    def verify(self, data: str, signature: str, public_key=None, padding_scheme='PSS') -> bool:
        """
        使用RSA算法验证签名
        
        :param data: 原始数据
        :param signature: Base64编码的签名
        :param public_key: PEM格式的公钥（可选，如果未提供则使用实例密钥）
        :param padding_scheme: 填充方案 ('PSS', 'PKCS1')
        :return: 如果验证成功返回True，否则返回False
        """
        if not public_key and not self.public_key_obj:
            raise ValueError("Public key is required for verification")
            
        if public_key:
            if isinstance(public_key, str):
                public_key = public_key.encode('utf-8')
            public_key_obj = serialization.load_pem_public_key(public_key)
            if not isinstance(public_key_obj, RSAPublicKey):
                raise ValueError("Provided key is not an RSA public key")
        else:
            public_key_obj = self.public_key_obj
            
        # Decode base64 signature
        signature_bytes = base64.b64decode(signature)
            
        # Verify the signature
        if public_key_obj is None:
            raise ValueError("No valid public key available for verification")
            
        # Apply padding scheme
        if padding_scheme == 'PSS':
            pad = padding.PSS(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                salt_length=padding.PSS.MAX_LENGTH
            )
            algorithm = hashes.SHA256()
        elif padding_scheme == 'PKCS1':
            pad = padding.PKCS1v15()
            algorithm = hashes.SHA256()
        else:
            raise ValueError("Unsupported padding scheme. Use 'PSS' or 'PKCS1'")
            
        try:
            public_key_obj.verify(signature_bytes, data.encode('utf-8'), pad, algorithm)
            return True
        except Exception:
            return False


# 便捷函数
def rsa_generate_keypair(key_size=1024):
    """
    生成新的RSA密钥对
    
    :param key_size: 密钥大小（比特），默认1024位
    :return: PEM格式的(私钥, 公钥)元组
    """
    return RSACipher.generate_keypair(key_size)


def rsa_encrypt_string(plaintext: str, public_key: str, padding_scheme='OAEP') -> str:
    """
    使用RSA加密字符串
    
    :param plaintext: 要加密的文本
    :param public_key: PEM格式的公钥
    :param padding_scheme: 填充方案 ('OAEP', 'PKCS1', 'NONE')
    :return: Base64编码的加密密文
    """
    cipher = RSACipher(public_key=public_key)
    return cipher.encrypt(plaintext, padding_scheme=padding_scheme)





def rsa_decrypt_with_public_key(ciphertext: str, public_key: str, padding_scheme='OAEP') -> str:
    """
    使用RSA公钥解密字符串

    :param ciphertext: Base64编码的密文
    :param public_key: PEM格式的公钥
    :param padding_scheme: 填充方案 ('OAEP', 'PKCS1')
    :return: 解密后的明文
    """
    cipher = RSACipher(public_key=public_key)
    return cipher.decrypt_with_public_key(ciphertext, padding_scheme=padding_scheme)


def rsa_decrypt_string(ciphertext: str, private_key: str, padding_scheme='OAEP') -> str:
    """
    使用RSA解密字符串
    
    :param ciphertext: Base64编码的密文
    :param private_key: PEM格式的私钥
    :param padding_scheme: 填充方案 ('OAEP', 'PKCS1', 'NONE')
    :return: 解密后的明文
    """
    cipher = RSACipher(private_key=private_key)
    return cipher.decrypt(ciphertext, padding_scheme=padding_scheme)


def rsa_sign(data: str, private_key: str, padding_scheme='PSS') -> str:
    """
    使用RSA算法对数据进行签名
    
    :param data: 要签名的数据
    :param private_key: PEM格式的私钥
    :param padding_scheme: 填充方案 ('PSS', 'PKCS1')
    :return: Base64编码的签名
    """
    cipher = RSACipher(private_key=private_key)
    return cipher.sign(data, padding_scheme=padding_scheme)


def rsa_verify(data: str, signature: str, public_key: str, padding_scheme='PSS') -> bool:
    """
    使用RSA算法验证签名
    
    :param data: 原始数据
    :param signature: Base64编码的签名
    :param public_key: PEM格式的公钥
    :param padding_scheme: 填充方案 ('PSS', 'PKCS1')
    :return: 如果验证成功返回True，否则返回False
    """
    cipher = RSACipher(public_key=public_key)
    return cipher.verify(data, signature, padding_scheme=padding_scheme)

if __name__ == '__main__':
    # Generate a new RSA key pair
    private_key, public_key = rsa_generate_keypair()
    print(f"Private key:\n{private_key}")
    print(f"Public key:\n{public_key}")

    plaintext = "hello"
    
    # Test with OAEP padding (default)
    print("\n--- Testing with OAEP padding ---")
    ciphertext = rsa_encrypt_string(plaintext, public_key, padding_scheme='OAEP')
    print(f"Ciphertext (OAEP): {ciphertext}")
    decrypted_text = rsa_decrypt_string(ciphertext, private_key, padding_scheme='OAEP')
    print(f"Decrypted (OAEP): {decrypted_text}")
    
    # Test with PKCS1 padding
    print("\n--- Testing with PKCS1 padding ---")
    ciphertext = rsa_encrypt_string(plaintext, public_key, padding_scheme='PKCS1')
    print(f"Ciphertext (PKCS1): {ciphertext}")
    decrypted_text = rsa_decrypt_string(ciphertext, private_key, padding_scheme='PKCS1')
    print(f"Decrypted (PKCS1): {decrypted_text}")

    # Test with PKCS1 padding
    print("\n--- Testing with NONE padding ---")
    ciphertext = rsa_encrypt_string(plaintext, public_key, padding_scheme='NONE')
    print(f"Ciphertext (NONE): {ciphertext}")
    decrypted_text = rsa_decrypt_string(ciphertext, private_key, padding_scheme='NONE')
    print(f"Decrypted (NONE): {decrypted_text}")

    # Sign a string
    signature = rsa_sign(plaintext, private_key, padding_scheme='PSS')
    print(f"Signature: {signature}")
    
    # Verify the signature
    is_valid = rsa_verify(plaintext, signature, public_key, padding_scheme='PSS')
    print(f"Signature valid: {is_valid}")


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="darren_utils",
    version="0.2.1.201",
    author="Darren",
    author_email="2775856@qq.com",
    description="一个功能丰富的工具包",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Darren5211314",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.7",
    install_requires=[
        "setuptools",
        "requests",
        "pyarmor>=9.0",
        "cryptography",
        "gmssl",
        "send2trash",
        "pyperclip",
        "psutil",
        "pillow",
    ],
)



import datetime
import json
import os
import re
from http.cookies import SimpleCookie

import requests
from urllib.parse import urlparse

from requests.cookies import RequestsCookieJar


def json_parse_safe(obj):
    """安全的JSON处理"""
    try:
        if isinstance(obj, str):
            return json.loads(obj) if obj else {}
        elif isinstance(obj, (dict, list)):
            # 如果已经是 dict 或 list，直接返回
            return obj
        else:
            return {}
    except (json.JSONDecodeError, TypeError):
        return {}
def get_public_ip(ip="",source=10):

    """
    获取公网IP信息
    :param ip: IP地址
    :param source: 获取IP信息的源，默认为10 0-15
    :return: IP信息字典
    """
    if source==0:
        url = f"https://whois.pconline.com.cn/ipJson.jsp?ip={ip}&json=true"
        ret = requests.get(url)
        if ret is None:
            return ""
        print(ret.text)
        json_data = json.loads(ret.text)

        ip_info = {
            "ip": json_data.get("ip"),
            "country": json_data.get("country"),
            "province": json_data.get("pro"),
            "city": json_data.get("city"),
            "isp": json_data.get("company"),
            "cityCode": json_data.get("cityCode"),
            "type": json_data.get("type"),
        }
        return ip_info
    elif source==1:
        url = f"https://api.vore.top/api/IPdata?ip={ip}"
        ret = requests.get(url)
        if ret is None:
            return ""
        print(ret.text)
        json_data = json.loads(ret.text)
        ip_info = {
            "ip": json_data.get("ipinfo", {}).get("text", ""),
            "country": json_data.get("country"),
            "province": json_data.get("ipdata", {}).get("info1", ""),
            "city": json_data.get("ipdata", {}).get("info2", ""),
            "district": json_data.get("ipdata", {}).get("info3", ""),  # 添加区/县信息
            "isp":json_data.get("ipdata", {}).get("isp", ""),
            "cityCode": json_data.get("adcode", {}).get("a", ""),
            "type": json_data.get("ipinfo", {}).get("type", ""),
        }
        return ip_info
    elif source==2:
        url = f"http://demo.ip-api.com/json/?lang=zh-CN"
        ret = requests.get(url)
        if ret is None:
            return ""
        print(ret.text)
        json_data = json.loads(ret.text)
        #{"status":"success","country":"中国","countryCode":"CN","region":"HA","regionName":"河南","city":"濮阳县城关镇","zip":"","lat":35.7062,"lon":115.028,"timezone":"Asia/Shanghai","isp":"Chinanet","org":"","as":"AS4134 CHINANET-BACKBONE","query":"123.52.205.93"}
        ip_info={
            "ip": json_data.get("query", ""),
            "country": json_data.get("country", ""),
            "province": json_data.get("regionName", ""),
            "city": json_data.get("city", ""),
            "district": json_data.get("district", ""),  # 添加区/县信息
            "isp": json_data.get("isp", ""),
            "cityCode": json_data.get("cityCode", ""),
            "type": json_data.get("type", ""),
            "lat": json_data.get("lat", ""),
            "lon": json_data.get("lon", ""),
        }
        return ip_info
    elif source==3:
        url = f"http://httpbin.org/ip"
        ret = requests.get(url)
        if ret is None:
            return ""
        json_data = json.loads(ret.text)
        #{"origin": "123.52.205.93"}
        ip_info = {
            "ip": json_data.get("origin", ""),
        }
        return ip_info
    elif source==4:
        url = f"https://vv.video.qq.com/checktime?otype=ojson"
        ret = requests.get(url)
        if ret is None:
            return ""
        print(ret.text)
        json_data = json.loads(ret.text)
        #{"s":"o","t":1764669188,"ip":"123.52.205.93","pos":"---","rand":"Xkgh_260PzliI2E-amN0zA=="}
        ip_info = {
            "ip": json_data.get("ip", ""),
        }
        return ip_info
    elif source==5:
        url = f"https://ipv4.my.ipinfo.app/api/ipDetails.php"
        ret = requests.get(url)
        if ret is None:
            return ""
        print(ret.text)
        json_data = json.loads(ret.text)
        #{"ip":"123.52.205.93","asn":"AS4134 - CHINANET-BACKBONE No.31,Jin-rong Street","continent":"AS","continentLong":"Asia","flag":"https://my.ipinfo.app/imgs/flags/4x3/cn.svg","country":"China"}
        ip_info = {
            "ip": json_data.get("ip", ""),
            "country": json_data.get("country", ""),
        }
        return ip_info
    elif source==6:
        url = f"https://r.inews.qq.com/api/ip2city"
        ret = requests.get(url)
        if ret is None:
            return ""
        json_data = json.loads(ret.text)
        #{"ret":0,"errMsg":"","ip":"123.52.205.93","provcode":"18","citycode":"188","country":"中国","province":"河南省","city":"濮阳市","district":"","isp":"","districtCode":"410900","callback":""}
        ip_info = {
            "ip": json_data.get("ip", ""),
            "country": json_data.get("country", ""),
            "province": json_data.get("province", ""),
            "city": json_data.get("city", ""),
            "district": json_data.get("district", ""),
            "isp": json_data.get("isp", ""),
            "cityCode": json_data.get("districtCode", ""),
        }
        return ip_info
    elif source==7:
        url = f"https://myip.ipip.net/json"
        ret = requests.get(url)
        if ret is None:
            return ""
        json_data = json.loads(ret.text)
        #{"ret":"ok","data":{"ip":"123.52.205.93","location":["中国","河南","濮阳","","电信"]}}
        ip_info = {
            "ip": json_data.get("data", {}).get("ip", ""),
            "country": json_data.get("data", {}).get("location", [])[0],
            "province": json_data.get("data", {}).get("location", [])[1],
            "city": json_data.get("data", {}).get("location", [])[2],
            "district": json_data.get("data", {}).get("location", [])[3],
            "isp": json_data.get("data", {}).get("location", [])[4],
        }
        return ip_info
    elif source==8:
        url = f"https://iplark.com/ipstack"
        ret = requests.get(url)
        if ret is None:
            return ""
        json_data = json.loads(ret.text)
        #{"ip":"123.52.205.93","type":"ipv4","continent_code":"AS","continent_name":"亚洲","country_code":"CN","country_name":"中国","region_code":"HA","region_name":"Henan","city":"Puyang","zip":"457000","latitude":35.6966705322266,"longitude":115.013893127441,"msa":null,"dma":null,"radius":"0","ip_routing_type":"fixed","connection_type":"tx","location":{"geoname_id":1798422,"capital":"Beijing","languages":[{"code":"zh","name":"Chinese","native":"中文"}],"country_flag":"https://assets.ipstack.com/flags/cn.svg","country_flag_emoji":"🇨🇳","country_flag_emoji_unicode":"U+1F1E8 U+1F1F3","calling_code":"86","is_eu":false},"time_zone":{"id":"Asia/Shanghai","current_time":"2025-12-02T18:12:02+08:00","gmt_offset":28800,"code":"CST","is_daylight_saving":false},"currency":{"code":"CNY","name":"Chinese Yuan","plural":"Chinese yuan","symbol":"CN¥","symbol_native":"CN¥"},"connection":{"asn":4134,"isp":"Chinanet","sld":null,"tld":null,"carrier":"chinanet","home":false,"organization_type":"Telecommunications","isic_code":"J6100","naics_code":"000517"},"security":{"is_proxy":false,"proxy_type":null,"is_crawler":false,"crawler_name":null,"crawler_type":null,"is_tor":false,"threat_level":"low","threat_types":null,"proxy_last_detected":null,"proxy_level":null,"vpn_service":null,"anonymizer_status":null,"hosting_facility":false}}
        ip_info = {
            "ip": json_data.get("ip", ""),
            "country": json_data.get("country_name", ""),
            "province": json_data.get("region_name", ""),
            "city": json_data.get("city", ""),
            "district": json_data.get("district", ""),
            "isp": json_data.get("isp", ""),
            "cityCode": json_data.get("city_code", ""),
            "type": json_data.get("type", ""),
            "lat": json_data.get("latitude", ""),
            "lon": json_data.get("longitude", ""),
        }
        return ip_info
    elif source==9:
        url = f"https://ipservice.ws.126.net/locate/api/getLocByIp"
        ret = requests.get(url)
        if ret is None:
            return ""
        json_data = json.loads(ret.text)
        #{"message":"查询成功","status":200,"result":{"administrativeCode":"410900","areaCode":"86","areaLat":"35.747699","areaLng":"115.014198","city":"濮阳","company":"电信","continentCode":"AP","country":"中国","countrySymbol":"CN","ip":"123.52.205.93","network":"AP","operator":"","province":"河南","timezone":"Asia/Shanghai","utc":"UTC+8"}}
        ip_info = {
            "ip": json_data.get("result", {}).get("ip", ""),
            "country": json_data.get("result", {}).get("country", ""),
            "province": json_data.get("result", {}).get("province", ""),
            "city": json_data.get("result", {}).get("city", ""),
            "district": json_data.get("result", {}).get("district", ""),
            "isp": json_data.get("result", {}).get("operator", ""),
            "cityCode": json_data.get("result", {}).get("administrativeCode", ""),
            "type": json_data.get("result", {}).get("network", ""),
            "lat": json_data.get("result", {}).get("areaLat", ""),
            "lon": json_data.get("result", {}).get("areaLng", ""),
        }
        return ip_info
    elif source==10:
        url = f"https://126.com/fgw/mailsrv-ipdetail/detail"
        ret = requests.get(url)
        if ret is None:
            return ""
        json_data = json.loads(ret.text)
        #{"code":200,"desc":"DONE","success":"false","result":{"country":"中国","province":"河南省","provinceEn":"Henan","city":"濮阳市","org":"中国电信","isp":"电信","latitude":"35.7532978882","longitude":"115.026627441","timezone":"Asia/Shanghai","countryCode":"CN","continentCode":"AS","provinceCode":"41","continent":"亚洲","county":"UNKNOWN","ispId":"10000","ip":"123.52.205.93","zone":"gz"}}
        ip_info = {
            "ip": json_data.get("result", {}).get("ip", ""),
            "country": json_data.get("result", {}).get("country", ""),
            "province": json_data.get("result", {}).get("province", ""),
            "city": json_data.get("result", {}).get("city", ""),
            "district": json_data.get("result", {}).get("county", ""),
            "isp": json_data.get("result", {}).get("isp", ""),
            "cityCode": json_data.get("result", {}).get("cityCode", ""),
            "lat": json_data.get("result", {}).get("latitude", ""),
            "lon": json_data.get("result", {}).get("longitude", ""),

        }
        return ip_info
    elif source==11:
        url = f"http://only-162333-112-96-112-201.nstool.zhuanzfx.com/info.js"
        ret = requests.get(url)
        if ret is None:
            return ""
        #var ip = '123.52.205.93'; var dns = '171.15.161.125'; var ip_province = '河南省'; var ip_city = '濮阳市'; var ip_isp = '电信'; var dns_province = '河南省'; var dns_city = '郑州市'; var dns_isp = '电信'; var res = 'correct'; var msg = '您的DNS设置正确';
        ip_info = {
            "ip": ret.text.split("ip = '")[1].split("';")[0],
            "province": ret.text.split("ip_province = '")[1].split("';")[0],
            "city": ret.text.split("ip_city = '")[1].split("';")[0],
            "isp": ret.text.split("ip_isp = '")[1].split("';")[0],
        }
        return ip_info
    elif source==12:
        url = f"https://ipv4.gdt.qq.com/get_client_ip"
        ret = requests.get(url)
        if ret is None:
            return ""
        #123.52.205.93
        ip_info = {
            "ip": ret.text.strip(),

        }
        return ip_info
    elif source==13:
        url = f"http://fn.syyx.com/my_ip"
        ret = requests.get(url)
        if ret is None:
            return ""
        ip_info = {
            "ip": ret.text.strip(),

        }
        return ip_info
    elif source==14:
        url = f"https://www.uc.cn/ip"
        ret = requests.get(url)
        if ret is None:
            return ""
        #IP:33.50.238.112
        ip_info = {
            "ip": ret.text.split("IP:")[1].strip(),
        }
        return ip_info
    elif source==15:
        url = f"https://who.nie.163.com/"
        ret = requests.get(url)
        if ret is None:
            return ""
        #{"ip":"123.52.205.93","country":"CN"}
        json_data = json.loads(ret.text)
        ip_info = {
            "ip": json_data.get("ip", ""),
            "country": json_data.get("country", ""),
        }
        return ip_info
    #https://get.geojs.io/v1/ip/geo.json
    return None


def url_get_domain(url):
    """
    获取URL的域名

    Args:
        url (str): URL字符串

    Returns:
        str: 域名
    """
    parsed_url = urlparse(url)
    return parsed_url.netloc
def get_jsonp(text):
    """
    解析JSONP字符串并返回JSON对象

    Args:
        text (str): JSONP字符串

    Returns:
        dict: 解析后的JSON对象
    """
    jsonp_str = re.search(r"\((.*)\)", text, re.S).group(1)
    return json.loads(jsonp_str)
def save_log(filename, log_content):
    """
    保存日志到指定文件

    :param filename: 日志文件名
    :param log_content: 需要保存的日志内容
    """
    # 定义日志文件夹路径
    log_dir = "logs"
    # 检查log文件夹是否存在，不存在则创建
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    # 构建完整的文件路径
    file_path = os.path.join(log_dir, filename)
    current_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    # 以追加模式打开文件，如果文件不存在会自动创建
    with open(file_path+".txt", 'a', encoding='utf-8') as file:
        file.write(f"[{current_time}] {log_content}\n")

def cookie_dict_to_string(cookie_dict):
    """
    将字典格式的cookie转换为字符串格式(key=value; key=value)

    Args:
        cookie_dict (dict): cookie字典

    Returns:
        str: 字符串格式的cookie
    """
    if not isinstance(cookie_dict, dict):
        raise ValueError("输入必须是字典类型")

    cookie_items = []
    for key, value in cookie_dict.items():
        cookie_items.append(f"{key}={value}")

    return "; ".join(cookie_items)
def cookie_string_to_dict(cookie_string):
    """
    将字符串格式的cookie转换为字典格式

    Args:
        cookie_string (str): 字符串格式的cookie (key=value; key=value)

    Returns:
        dict: cookie字典
    """
    if not isinstance(cookie_string, str):
        raise ValueError("输入必须是字符串类型")

    cookie_dict = {}
    if not cookie_string.strip():
        return cookie_dict

    # 按分号分割cookie项
    items = cookie_string.split(";")
    for item in items:
        item = item.strip()  # 去除前后空格
        if "=" in item:
            key, value = item.split("=", 1)  # 只分割第一个等号
            cookie_dict[key.strip()] = value.strip()

    return cookie_dict


def cookie_to_dict(cookie):
    """
    内部工具函数：将任意支持的 Cookie 类型转换为标准字典
    支持类型：str（普通key=value/Set-Cookie头）、dict、RequestsCookieJar
    """
    cookie_dict = {}

    # 1. 处理字典类型
    if isinstance(cookie, dict):
        # 过滤空值
        cookie_dict = {k: v for k, v in cookie.items() if v not in (None, '')}

    # 2. 处理 RequestsCookieJar（response.cookies 类型）
    elif isinstance(cookie, RequestsCookieJar):
        # 遍历所有Cookie条目，仅保留非空值，后出现的非空值覆盖前值
        for morsel in cookie:
            if morsel.value not in (None, ''):  # 过滤空值
                cookie_dict[morsel.name] = morsel.value

    # 3. 处理字符串类型（兼容两种格式）
    elif isinstance(cookie, str):
        # 3.1 处理包含 Set-Cookie 标识的响应头字符串
        if 'Set-Cookie:' in cookie:
            set_cookie_lines = [
                line.strip().replace('Set-Cookie:', '').strip()
                for line in cookie.split('\n')
                if line.strip().startswith('Set-Cookie:')
            ]
            for line in set_cookie_lines:
                try:
                    sc = SimpleCookie()
                    sc.load(line)
                    for key, morsel in sc.items():
                        if morsel.value not in (None, ''):  # 过滤空值
                            cookie_dict[key] = morsel.value
                except Exception:
                    match = re.match(r'^([^=]+)=([^;]+)', line.strip())
                    if match:
                        key = match.group(1).strip()
                        value = match.group(2).strip()
                        if value not in (None, ''):  # 过滤空值
                            cookie_dict[key] = value
        # 3.2 处理普通 Cookie 字符串
        else:
            for item in cookie.split(';'):
                item = item.strip()
                if not item or '=' not in item:
                    continue
                key, value = item.split('=', 1)
                key = key.strip()
                value = value.strip()
                if value not in (None, ''):  # 过滤空值
                    cookie_dict[key] = value

    # 4. 不支持的类型抛出异常
    else:
        raise TypeError(
            f"不支持的 Cookie 类型：{type(cookie)}，仅支持 str/字典/RequestsCookieJar（response.cookies）"
        )

    return cookie_dict


def cookie_merge(old_cookie, new_cookie):
    try:
        old_type = type(old_cookie)
        old_dict = cookie_to_dict(old_cookie)
        new_dict = cookie_to_dict(new_cookie)

        # 核心合并逻辑：仅用新字典中的非空值覆盖旧字典
        merged_dict = old_dict.copy()
        for k, v in new_dict.items():
            if v not in (None, ''):  # 再次确认非空（双重保险）
                merged_dict[k] = v

        return _dict_to_original_type(merged_dict, old_type, old_cookie)

    except Exception as e:
        raise ValueError(f"Cookie 合并失败：{str(e)}") from e
def _dict_to_original_type(merged_dict, original_type, original_obj=None):
    """
    内部工具函数：将合并后的字典转回旧 Cookie 的原始类型
    """
    # 转回字典
    if original_type is dict:
        return merged_dict

    # 转回 RequestsCookieJar（response.cookies 类型）
    elif original_type is RequestsCookieJar:
        jar = RequestsCookieJar()
        jar.update(merged_dict)
        return jar

    # 转回字符串（标准 key=value; 格式）
    elif original_type is str:
        return '; '.join([f"{k}={v}" for k, v in merged_dict.items() if v is not None])

    # 不支持的转换类型
    else:
        raise TypeError(f"无法转换为目标类型：{original_type}")

if __name__ == '__main__':
    print(get_public_ip(source=10))

    pass





import hashlib
import zlib


# ... 现有的哈希函数 ...

def hash_crc32_string(text: str) -> str:
    """
    CRC32校验，返回8位小写十六进制字符串

    Args:
        text (str): 要计算CRC32的文本

    Returns:
        str: 8位小写CRC32值
    """
    crc32_value = zlib.crc32(text.encode('utf-8')) & 0xffffffff
    return format(crc32_value, '08x')


def hash_crc32_int(text: str) -> int:
    """
    CRC32校验，返回整数值

    Args:
        text (str): 要计算CRC32的文本

    Returns:
        int: CRC32整数值
    """
    return zlib.crc32(text.encode('utf-8')) & 0xffffffff


class RC4:
    """
    RC4加密算法实现
    """

    def __init__(self, key: str):
        """
        初始化RC4算法

        Args:
            key (str): 加密密钥
        """
        self.key = key.encode('utf-8')
        self.S = self._ksa()

    def _ksa(self):
        """
        密钥调度算法(Key Scheduling Algorithm)

        Returns:
            list: 初始化的S盒
        """
        key_length = len(self.key)
        S = list(range(256))
        j = 0
        for i in range(256):
            j = (j + S[i] + self.key[i % key_length]) % 256
            S[i], S[j] = S[j], S[i]
        return S

    def _prga(self):
        """
        伪随机生成算法(Pseudo-Random Generation Algorithm)

        Yields:
            int: 生成的密钥流字节
        """
        i = 0
        j = 0
        S = self.S[:]
        while True:
            i = (i + 1) % 256
            j = (j + S[i]) % 256
            S[i], S[j] = S[j], S[i]
            K = S[(S[i] + S[j]) % 256]
            yield K

    def encrypt(self, plaintext: str) -> bytes:
        """
        RC4加密

        Args:
            plaintext (str): 明文

        Returns:
            bytes: 加密后的字节数据
        """
        keystream = self._prga()
        plaintext_bytes = plaintext.encode('utf-8')
        ciphertext = bytearray()
        for byte in plaintext_bytes:
            ciphertext.append(byte ^ next(keystream))
        return bytes(ciphertext)

    def decrypt(self, ciphertext: bytes) -> str:
        """
        RC4解密

        Args:
            ciphertext (bytes): 密文

        Returns:
            str: 解密后的明文
        """
        # RC4加密和解密使用相同的过程
        keystream = self._prga()
        plaintext = bytearray()
        for byte in ciphertext:
            plaintext.append(byte ^ next(keystream))
        return plaintext.decode('utf-8')


def rc4_encrypt_string(plaintext: str, key: str) -> str:
    """
    RC4加密，返回十六进制字符串

    Args:
        plaintext (str): 明文
        key (str): 加密密钥

    Returns:
        str: 十六进制格式的密文
    """
    rc4 = RC4(key)
    ciphertext = rc4.encrypt(plaintext)
    return ciphertext.hex()


def rc4_decrypt_string(ciphertext_hex: str, key: str) -> str:
    """
    RC4解密十六进制字符串

    Args:
        ciphertext_hex (str): 十六进制格式的密文
        key (str): 解密密钥

    Returns:
        str: 解密后的明文
    """
    rc4 = RC4(key)
    ciphertext = bytes.fromhex(ciphertext_hex)
    return rc4.decrypt(ciphertext)



import hashlib
import zlib


def hash_md5_string(text):
    """
    MD5加密，返回32位小写十六进制字符串

    Args:
        text (str): 要加密的文本

    Returns:
        str: 32位小写MD5值

    Example:
        >>> hash_md5_string("hello")
        '5d41402abc4b2a76b9719d911017c592'
    """
    md5_hash = hashlib.md5()
    md5_hash.update(text.encode('utf-8'))
    return md5_hash.hexdigest()


def hash_md5_bytes(text: str) -> bytes:
    """
    MD5加密，返回字节格式

    Args:
        text (str): 要加密的文本

    Returns:
        bytes: MD5字节值
    """
    md5_hash = hashlib.md5()
    md5_hash.update(text.encode('utf-8'))
    return md5_hash.digest()


def hash_sha1_string(text: str) -> str:
    """
    SHA1加密，返回40位小写十六进制字符串

    Args:
        text (str): 要加密的文本

    Returns:
        str: 40位小写SHA1值
    """
    sha1_hash = hashlib.sha1()
    sha1_hash.update(text.encode('utf-8'))
    return sha1_hash.hexdigest()


def hash_sha256_string(text: str) -> str:
    """
    SHA256加密，返回64位小写十六进制字符串

    Args:
        text (str): 要加密的文本

    Returns:
        str: 64位小写SHA256值
    """
    sha256_hash = hashlib.sha256()
    sha256_hash.update(text.encode('utf-8'))
    return sha256_hash.hexdigest()


def hash_sha512_string(text: str) -> str:
    """
    SHA512加密，返回128位小写十六进制字符串

    Args:
        text (str): 要加密的文本

    Returns:
        str: 128位小写SHA512值
    """
    sha512_hash = hashlib.sha512()
    sha512_hash.update(text.encode('utf-8'))
    return sha512_hash.hexdigest()


def hash_sha1_bytes(text: str) -> bytes:
    """
    SHA1加密，返回字节格式

    Args:
        text (str): 要加密的文本

    Returns:
        bytes: SHA1字节值
    """
    sha1_hash = hashlib.sha1()
    sha1_hash.update(text.encode('utf-8'))
    return sha1_hash.digest()


def hash_sha256_bytes(text: str) -> bytes:
    """
    SHA256加密，返回字节格式

    Args:
        text (str): 要加密的文本

    Returns:
        bytes: SHA256字节值
    """
    sha256_hash = hashlib.sha256()
    sha256_hash.update(text.encode('utf-8'))
    return sha256_hash.digest()


def hash_sha512_bytes(text: str) -> bytes:
    """
    SHA512加密，返回字节格式

    Args:
        text (str): 要加密的文本

    Returns:
        bytes: SHA512字节值
    """
    sha512_hash = hashlib.sha512()
    sha512_hash.update(text.encode('utf-8'))
    return sha512_hash.digest()


def hash_crc32_string(text: str) -> str:
    """
    CRC32校验，返回8位小写十六进制字符串

    Args:
        text (str): 要计算CRC32的文本

    Returns:
        str: 8位小写CRC32值
    """
    crc32_value = zlib.crc32(text.encode('utf-8')) & 0xffffffff
    return format(crc32_value, '08x')


def hash_crc32_int(text: str) -> int:
    """
    CRC32校验，返回整数值

    Args:
        text (str): 要计算CRC32的文本

    Returns:
        int: CRC32整数值
    """
    return zlib.crc32(text.encode('utf-8')) & 0xffffffff
if __name__ == '__main__':
    crc32_hex = hash_crc32_string("1")  # '3610a686'
    crc32_int = hash_crc32_int("hello")  # 906962566
    print(crc32_hex, crc32_int)
